import 'package:layout_form_builder/datalayer/constant/eform_constants.dart';
import 'package:http/http.dart' as http;
import 'eform_source.dart';

typedef EFormDataLoader = Future<Map<String, dynamic>> Function();

class PageJsonFormData {
  final String pageId;
  final String pageJson;
  final Map<String, dynamic> formData;

  PageJsonFormData({required this.pageId, required this.pageJson, required this.formData});

  static Future<PageJsonFormData> from(FormSource source, EFormDataLoader dataLoader, String pageId) async {
    String finalJsonStr = "";
    if (source.jsonStr != null && source.jsonStr!.isNotEmpty) {
      finalJsonStr = source.jsonStr!;
    } else if (source.url != null && source.url!.isNotEmpty) {
      final response = await http.get(Uri.parse(source.url!.replaceAll(EFormConstants.FORM_PAGE_ID_VAR, pageId)));

      //The http response is successful based on status code
      if (response.statusCode >= 200 && response.statusCode < 300) {
        finalJsonStr = response.body;
        print("[PageJsonFormData] Final form json: $finalJsonStr");
      } else {
        throw Exception('[PageJsonFormData] Failed to load json data, use to populate the FORM UI.');
      }

    } else {
      throw Exception('[PageJsonFormData] Please specify the form source');
    }
    return PageJsonFormData(
      pageId: pageId,
      pageJson: finalJsonStr,
      formData: await dataLoader()
    );
  }
}