import 'package:dynamic_widget/dynamic_widget.dart';
import 'package:flutter/material.dart';
import 'package:flutter_form_builder_ing/flutter_form_builder_ing.dart';
import 'package:layout_form_builder/layout_form_renderer.dart';
import 'package:layout_form_builder/ui/app_colors.dart';
import 'datalayer/bean/eform_page_json_form_data.dart';
import 'datalayer/bean/eform_source.dart';

typedef EFormOnValidateCallback = bool Function();
typedef EFormOnValueCallback = Map<String, dynamic> Function();

class EFormDynamicLayoutController{
  late Function _reset;
  late Function _save;
  late EFormOnValidateCallback _validate;
  late EFormOnValueCallback _value;

  void reset(){
    _reset();
  }

  void save(){
    _save();
  }

  bool validate(){
    return _validate();
  }

  Map<String, dynamic> value(){
    return _value();
  }
}

class EFormDynamicLayout extends StatefulWidget {
  final FormSource source;
  final EFormDataLoader dataLoader;
  final String pageId;
  final GlobalKey<FormBuilderState>? formKey;
  final EFormDynamicLayoutController? controller;

  EFormDynamicLayout({Key? key, required this.source, this.controller, required this.dataLoader, required this.pageId, this.formKey}) : super(key: key);

  @override
  _EFormDynamicLayoutState createState() => _EFormDynamicLayoutState();
}

class _EFormDynamicLayoutState extends State<EFormDynamicLayout> {
  late Future<PageJsonFormData> pageJsonFormData;
  late EFormDynamicLayoutController controller;
  late GlobalKey<FormBuilderState> formKey;

  @override
  void initState() {
    super.initState();
    _loadPageUI();
    formKey = widget.formKey ?? GlobalKey();
    controller = widget.controller ?? EFormDynamicLayoutController();
    controller._reset = (){
      formKey.currentState?.reset();
    };
    controller._save = (){
      formKey.currentState?.save();
    };
    controller._validate = (){
      return formKey.currentState?.validate() ?? false;
    };
    controller._value = (){
      return formKey.currentState?.value ?? Map();
    };
  }

  void _loadPageUI() {
    pageJsonFormData = PageJsonFormData.from(widget.source, widget.dataLoader, widget.pageId);
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<PageJsonFormData>(
      future: pageJsonFormData,
      builder: (context, snapshot) {
        if (snapshot.hasData) {
          return LayoutForm(
            formKey: formKey,
            clickListener: DefaultClickListener(),
            pageJsonFormData: snapshot.data!
          );
        } else if (snapshot.hasError) {
          return Text('Error: ${snapshot.error}', style: TextStyle(color: AppColors.TextGray, fontSize: 14));
        }
        return Container(
          width: 30,
          height: 30,
          child: CircularProgressIndicator(valueColor: new AlwaysStoppedAnimation<Color>(AppColors.AccentBright), strokeWidth: 2.0, backgroundColor: AppColors.AccentBright.withOpacity(0.4))
        );
      });
  }
}

class DefaultClickListener implements ClickListener {
  @override
  void onClicked(String? event) {}
}
